=== modified file '.bzrignore'
--- .bzrignore	2005-11-25 04:36:54 +0000
+++ .bzrignore	2005-12-09 21:48:26 +0000
@@ -19,3 +19,4 @@
 ./ID
 ./tags
 .bzr-shelf
+BRANCH-INFO

=== modified file 'NEWS'
--- NEWS	2006-01-25 05:10:06 +0000
+++ NEWS	2006-01-31 15:33:47 +0000
@@ -1,5 +1,23 @@
 bzr development version
 
+  IMPROVEMENTS:
+
+    * The FTP transport now tries to reconnect after a temporary
+      failure. ftp put is made atomic. (Matthieu Moy)
+
+    * The FTP transport now maintains a pool of connections, and
+      reuses them to avoid multiple connections to the same host (like
+      sftp did). (Daniel Silverstone)
+
+    * The bzr_man.py file has been removed. To create the man page now,
+      use ./generate_docs.py man. The new program can also create other files.
+      Run "python generate_docs.py --help" for usage information. (Hans
+      Ulrich Niedermann & James Blackwell).
+
+    * Man Page now gives full help (James Blackwell). Help also updated to 
+      reflect user config now being stored in .bazaar (Hans Ulrich
+      Niedermann)
+
   INTERNALS:
   
     * The internal storage of history, and logical branch identity have now
@@ -117,9 +135,13 @@
     * 'bzr add' now takes a --dry-run parameter which shows you what would be
       added, but doesn't actually add anything. (Michael Ellerman)
 
-    * bzr add now lists how many files were ignored per glob.  add --verbose
+    * 'bzr add' now lists how many files were ignored per glob.  add --verbose
       lists the specific files.  (Aaron Bentley)
 
+    * 'bzr missing' now supports displaying changes in diverged trees and can
+      be limited to show what either end of the comparison is missing.
+      (Aaron Bently, with a little prompting from Daniel Silverstone)
+
   BUG FIXES:
 
     * SFTP can walk up to the root path without index errors. (Robert Collins)

=== modified file 'bzrlib/_changeset.py'
--- bzrlib/_changeset.py	2006-01-23 14:49:49 +0000
+++ bzrlib/_changeset.py	2006-01-31 14:52:42 +0000
@@ -647,7 +647,6 @@
 
         :rtype: bool
         """
-
         return (self.parent != self.new_parent or self.name != self.new_name)
 
     def is_deletion(self, reverse=False):

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2006-02-01 13:08:13 +0000
+++ bzrlib/builtins.py	2006-02-01 13:17:42 +0000
@@ -708,7 +708,7 @@
         tree = WorkingTree.open_containing(u'.')[0]
         b = tree.branch
         # FIXME. should be tree.last_revision
-        for revision_id in b.get_ancestry(b.last_revision()):
+        for revision_id in b.repository.get_ancestry(b.last_revision()):
             print revision_id
 
 
@@ -1170,8 +1170,8 @@
 
     Note: export of tree with non-ascii filenames to zip is not supported.
 
-    Supported formats       Autodetected by extension
-    -----------------       -------------------------
+     Supported formats       Autodetected by extension
+     -----------------       -------------------------
          dir                            -
          tar                          .tar
          tbz2                    .tar.bz2, .tbz2
@@ -1393,8 +1393,8 @@
 
 
 class cmd_nick(Command):
-    """\
-    Print or set the branch nickname.  
+    """Print or set the branch nickname.  
+
     If unset, the tree root directory name is used as the nickname
     To print the current nickname, execute with no argument.  
     """
@@ -1556,7 +1556,8 @@
         last1 = branch1.last_revision()
         last2 = branch2.last_revision()
 
-        source = MultipleRevisionSources(branch1, branch2)
+        source = MultipleRevisionSources(branch1.repository, 
+                                         branch2.repository)
         
         base_rev_id = common_ancestor(last1, last2, source)
 

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2006-02-01 12:24:35 +0000
+++ bzrlib/errors.py	2006-02-01 13:17:42 +0000
@@ -388,6 +388,16 @@
     """Text did not match it's checksum: %(message)s"""
 
 
+class WeaveTextDiffers(WeaveError):
+    """Weaves differ on text content. Revision: {%(revision_id)s}, %(weave_a)s, %(weave_b)s"""
+
+    def __init__(self, revision_id, weave_a, weave_b):
+        WeaveError.__init__(self)
+        self.revision_id = revision_id
+        self.weave_a = weave_a
+        self.weave_b = weave_b
+
+
 class NoSuchExportFormat(BzrNewError):
     """Export format %(format)r not supported"""
     def __init__(self, format):

=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2006-01-25 04:23:16 +0000
+++ bzrlib/fetch.py	2006-01-30 01:13:27 +0000
@@ -186,16 +186,20 @@
         self.pb.clear()
 
     def _fetch_inventory_weave(self, revs):
-        self.pb.update("inventory merge", 0, 1)
+        self.pb.update("inventory fetch", 0, 2)
         from_weave = self.from_repository.get_inventory_weave()
         to_weave = self.to_repository.get_inventory_weave()
+        self.pb.update("inventory fetch", 1, 2)
+        to_weave = self.to_control.get_weave('inventory',
+                self.to_branch.get_transaction())
+        self.pb.update("inventory fetch", 2, 2)
 
         if to_weave.numversions() > 0:
             # destination has contents, must merge
             try:
-                to_weave.join(from_weave)
+                to_weave.join(from_weave, pb=self.pb, msg='merge inventory')
             except errors.WeaveParentMismatch:
-                to_weave.reweave(from_weave)
+                to_weave.reweave(from_weave, pb=self.pb, msg='reweave inventory')
         else:
             # destination is empty, just replace it
             to_weave = from_weave.copy()

=== modified file 'bzrlib/store/text.py'
--- bzrlib/store/text.py	2005-12-15 21:11:09 +0000
+++ bzrlib/store/text.py	2006-01-25 17:56:27 +0000
@@ -95,7 +95,7 @@
             raise KeyError(fileid + '-' + str(suffix))
 
         try:
-            result = other._transport.copy_to([path], self._transport, pb=pb,
+            result = other._transport.copy_to([path], self._transport, 
                                               mode=self._file_mode)
         except NoSuchFile:
             if not self._prefixed:
@@ -104,7 +104,7 @@
                 self._transport.mkdir(hash_prefix(fileid)[:-1], mode=self._dir_mode)
             except FileExists:
                 pass
-            result = other._transport.copy_to([path], self._transport, pb=pb,
+            result = other._transport.copy_to([path], self._transport,
                                               mode=self._file_mode)
 
         if result != 1:

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2006-02-01 12:24:35 +0000
+++ bzrlib/tests/__init__.py	2006-02-01 13:17:42 +0000
@@ -815,8 +815,9 @@
                    'bzrlib.tests.test_commit_merge',
                    'bzrlib.tests.test_config',
                    'bzrlib.tests.test_conflicts',
+                   'bzrlib.tests.test_decorators',
                    'bzrlib.tests.test_diff',
-                   'bzrlib.tests.test_decorators',
+                   'bzrlib.tests.test_doc_generate',
                    'bzrlib.tests.test_fetch',
                    'bzrlib.tests.test_fileid_involved',
                    'bzrlib.tests.test_gpg',

=== modified file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	2006-01-26 20:52:58 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-02-01 13:17:42 +0000
@@ -29,9 +29,11 @@
 def test_suite():
     testmod_names = [
                      'bzrlib.tests.blackbox.test_added',
+                     'bzrlib.tests.blackbox.test_ancestry',
                      'bzrlib.tests.blackbox.test_cat',
                      'bzrlib.tests.blackbox.test_diff',
                      'bzrlib.tests.blackbox.test_export',
+                     'bzrlib.tests.blackbox.test_find_merge_base',
                      'bzrlib.tests.blackbox.test_missing',
                      'bzrlib.tests.blackbox.test_outside_wt',
                      'bzrlib.tests.blackbox.test_pull',

=== added file 'bzrlib/tests/blackbox/test_ancestry.py'
--- bzrlib/tests/blackbox/test_ancestry.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_ancestry.py	2006-01-31 14:27:38 +0000
@@ -0,0 +1,41 @@
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.branch import Branch
+from bzrlib.builtins import merge
+
+class TestAncestry(TestCaseInTempDir):
+
+    def test_ancestry(self):
+        """Tests 'ancestry' command"""
+        os.mkdir('A')
+        a = Branch.initialize('A')
+        a_wt = a.working_tree()
+        open('A/foo', 'wb').write('1111\n')
+        a_wt.add('foo')
+        a_wt.commit('added foo',rev_id='A1')
+        a.clone('B')
+        b = Branch.open('B')
+        b_wt = b.working_tree()
+        open('B/foo','wb').write('1111\n22\n')
+        b_wt.commit('modified B/foo',rev_id='B1')
+        open('A/foo', 'wb').write('000\n1111\n')
+        a_wt.commit('modified A/foo',rev_id='A2')
+        merge(['B',-1],['B',1],this_dir='A')
+        a_wt.commit('merged B into A',rev_id='A3')
+        self.run_bzr('ancestry')

=== added file 'bzrlib/tests/blackbox/test_find_merge_base.py'
--- bzrlib/tests/blackbox/test_find_merge_base.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_find_merge_base.py	2006-01-31 14:24:22 +0000
@@ -0,0 +1,34 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+from bzrlib.tests.blackbox import ExternalBase
+import os
+
+
+class TestFindMergeBase(ExternalBase):
+    def test_find_merge_base(self):
+        os.mkdir('a')
+        os.chdir('a')
+        self.runbzr('init')
+        self.runbzr('commit -m foo --unchanged')
+        self.runbzr('branch . ../b')
+        q = self.run_bzr('find-merge-base', '.', '../b', backtick=True)
+        self.runbzr('commit -m bar --unchanged')
+        os.chdir('../b')
+        self.runbzr('commit -m baz --unchanged')
+        r = self.run_bzr('find-merge-base', '.', '../a', backtick=True)
+        self.assertEqual(q, r)
+        

=== added file 'bzrlib/tests/test_doc_generate.py'
--- bzrlib/tests/test_doc_generate.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_doc_generate.py	2006-01-31 15:51:02 +0000
@@ -0,0 +1,28 @@
+# Copyright (C) 2006 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.branch import Branch
+import tools.doc_generate
+
+
+class TestDocGenerate(TestCaseInTempDir):
+
+    def test_generate_manpage(self):
+        """Simple smoke test for doc_generate"""
+        infogen_mod = tools.doc_generate.get_module("man")
+
+

=== modified file 'bzrlib/tests/test_weave.py'
--- bzrlib/tests/test_weave.py	2005-12-20 02:45:27 +0000
+++ bzrlib/tests/test_weave.py	2006-01-31 14:09:11 +0000
@@ -991,3 +991,81 @@
         self.assertRaises(errors.WeaveInvalidChecksum, w.check)
 
 
+class InstrumentedWeave(Weave):
+    """Keep track of how many times functions are called."""
+    
+    def __init__(self, weave_name=None):
+        self._extract_count = 0
+        Weave.__init__(self, weave_name=weave_name)
+
+    def _extract(self, versions):
+        self._extract_count += 1
+        return Weave._extract(self, versions)
+
+
+class JoinOptimization(TestCase):
+    """Test that Weave.join() doesn't extract all texts, only what must be done."""
+
+    def test_join(self):
+        w1 = InstrumentedWeave()
+        w2 = InstrumentedWeave()
+
+        txt0 = ['a\n']
+        txt1 = ['a\n', 'b\n']
+        txt2 = ['a\n', 'c\n']
+        txt3 = ['a\n', 'b\n', 'c\n']
+
+        w1.add('txt0', [], txt0) # extract 1a
+        w2.add('txt0', [], txt0) # extract 1b
+        w1.add('txt1', [0], txt1)# extract 2a
+        w2.add('txt2', [0], txt2)# extract 2b
+        w1.join(w2) # extract 3a to add txt2 
+        w2.join(w1) # extract 3b to add txt1 
+
+        w1.add('txt3', [1, 2], txt3) # extract 4a 
+        w2.add('txt3', [1, 2], txt3) # extract 4b
+        # These secretly have inverted parents
+
+        # This should not have to do any extractions
+        w1.join(w2) # NO extract, texts already present with same parents
+        w2.join(w1) # NO extract, texts already present with same parents
+
+        self.assertEqual(4, w1._extract_count)
+        self.assertEqual(4, w2._extract_count)
+
+    def test_double_parent(self):
+        # It should not be considered illegal to add
+        # a revision with the same parent twice
+        w1 = InstrumentedWeave()
+        w2 = InstrumentedWeave()
+
+        txt0 = ['a\n']
+        txt1 = ['a\n', 'b\n']
+        txt2 = ['a\n', 'c\n']
+        txt3 = ['a\n', 'b\n', 'c\n']
+
+        w1.add('txt0', [], txt0)
+        w2.add('txt0', [], txt0)
+        w1.add('txt1', [0], txt1)
+        w2.add('txt1', [0,0], txt1)
+        # Same text, effectively the same, because the
+        # parent is only repeated
+        w1.join(w2) # extract 3a to add txt2 
+        w2.join(w1) # extract 3b to add txt1 
+
+
+class MismatchedTexts(TestCase):
+    """Test that merging two weaves with different texts fails."""
+
+    def test_reweave(self):
+        w1 = Weave('a')
+        w2 = Weave('b')
+
+        w1.add('txt0', [], ['a\n'])
+        w2.add('txt0', [], ['a\n'])
+        w1.add('txt1', [0], ['a\n', 'b\n'])
+        w2.add('txt1', [0], ['a\n', 'c\n'])
+
+        self.assertRaises(errors.WeaveTextDiffers, w1.reweave, w2)
+
+

=== modified file 'bzrlib/transport/ftp.py'
--- bzrlib/transport/ftp.py	2006-01-12 06:06:03 +0000
+++ bzrlib/transport/ftp.py	2006-01-31 09:28:14 +0000
@@ -31,13 +31,30 @@
 import urllib
 import urlparse
 import stat
+import time
+import random
 from warnings import warn
 
 
 from bzrlib.transport import Transport
 from bzrlib.errors import (TransportNotPossible, TransportError,
                            NoSuchFile, FileExists)
-from bzrlib.trace import mutter
+from bzrlib.trace import mutter, warning
+
+
+_FTP_cache = {}
+def _find_FTP(hostname, username, password, is_active):
+    """Find an ftplib.FTP instance attached to this triplet."""
+    key = "%s|%s|%s|%s" % (hostname, username, password, is_active)
+    if key not in _FTP_cache:
+        mutter("Constructing FTP instance against %r" % key)
+        _FTP_cache[key] = ftplib.FTP(hostname, username, password)
+        _FTP_cache[key].set_pasv(not is_active)
+    return _FTP_cache[key]    
+
+
+class FtpTransportError(TransportError):
+    pass
 
 
 class FtpStatResult(object):
@@ -54,6 +71,9 @@
                 f.cwd(pwd)
 
 
+_number_of_retries = 2
+_sleep_between_retries = 5
+
 class FtpTransport(Transport):
     """This is the transport agent for ftp:// access."""
 
@@ -69,7 +89,6 @@
             self._query, self._fragment) = urlparse.urlparse(self.base)
         self._FTP_instance = _provided_instance
 
-
     def _get_FTP(self):
         """Return the ftplib.FTP instance for this object."""
         if self._FTP_instance is not None:
@@ -84,9 +103,8 @@
             if ':' in username:
                 username, password = username.split(":", 1)
 
-            mutter("Constructing FTP instance")
-            self._FTP_instance = ftplib.FTP(hostname, username, password)
-            self._FTP_instance.set_pasv(not self.is_active)
+            self._FTP_instance = _find_FTP(hostname, username, password,
+                                           self.is_active)
             return self._FTP_instance
         except ftplib.error_perm, e:
             raise TransportError(msg="Error setting up connection: %s"
@@ -161,10 +179,12 @@
             mutter("FTP has not: %s" % self._abspath(relpath))
             return False
 
-    def get(self, relpath, decode=False):
+    def get(self, relpath, decode=False, retries=0):
         """Get the file at the given relative path.
 
         :param relpath: The relative path to the file
+        :param retries: Number of retries after temporary failures so far
+                        for this operation.
 
         We're meant to return a file-like object which bzr will
         then read from. For now we do this via the magic of StringIO
@@ -178,23 +198,79 @@
             return ret
         except ftplib.error_perm, e:
             raise NoSuchFile(self.abspath(relpath), extra=str(e))
+        except ftplib.error_temp, e:
+            if retries > _number_of_retries:
+                raise TransportError(msg="FTP temporary error during GET %s. Aborting."
+                                     % self.abspath(relpath),
+                                     orig_error=e)
+            else:
+                warning("FTP temporary error: %s. Retrying." % str(e))
+                self._FTP_instance = None
+                return self.get(relpath, decode, retries+1)
+        except EOFError, e:
+            if retries > _number_of_retries:
+                raise TransportError("FTP control connection closed during GET %s."
+                                     % self.abspath(relpath),
+                                     orig_error=e)
+            else:
+                warning("FTP control connection closed. Trying to reopen.")
+                time.sleep(_sleep_between_retries)
+                self._FTP_instance = None
+                return self.get(relpath, decode, retries+1)
 
-    def put(self, relpath, fp, mode=None):
+    def put(self, relpath, fp, mode=None, retries=0):
         """Copy the file-like or string object into the location.
 
         :param relpath: Location to put the contents, relative to base.
-        :param f:       File-like or string object.
-        TODO: jam 20051215 This should be an atomic put, not overwritting files in place
+        :param fp:       File-like or string object.
+        :param retries: Number of retries after temporary failures so far
+                        for this operation.
+
         TODO: jam 20051215 ftp as a protocol seems to support chmod, but ftplib does not
         """
+        tmp_abspath = '%s.tmp.%.9f.%d.%d' % (self._abspath(relpath), time.time(),
+                        os.getpid(), random.randint(0,0x7FFFFFFF))
         if not hasattr(fp, 'read'):
             fp = StringIO(fp)
         try:
             mutter("FTP put: %s" % self._abspath(relpath))
             f = self._get_FTP()
-            f.storbinary('STOR '+self._abspath(relpath), fp, 8192)
+            try:
+                f.storbinary('STOR '+tmp_abspath, fp)
+                f.rename(tmp_abspath, self._abspath(relpath))
+            except (ftplib.error_temp,EOFError), e:
+                warning("Failure during ftp PUT. Deleting temporary file.")
+                try:
+                    f.delete(tmp_abspath)
+                except:
+                    warning("Failed to delete temporary file on the server.\nFile: %s"
+                            % tmp_abspath)
+                    raise e
+                raise
         except ftplib.error_perm, e:
-            raise TransportError(orig_error=e)
+            if "no such file" in str(e).lower():
+                raise NoSuchFile("Error storing %s: %s"
+                                 % (self.abspath(relpath), str(e)), extra=e)
+            else:
+                raise FtpTransportError(orig_error=e)
+        except ftplib.error_temp, e:
+            if retries > _number_of_retries:
+                raise TransportError("FTP temporary error during PUT %s. Aborting."
+                                     % self.abspath(relpath), orig_error=e)
+            else:
+                warning("FTP temporary error: %s. Retrying." % str(e))
+                self._FTP_instance = None
+                self.put(relpath, fp, mode, retries+1)
+        except EOFError:
+            if retries > _number_of_retries:
+                raise TransportError("FTP control connection closed during PUT %s."
+                                     % self.abspath(relpath), orig_error=e)
+            else:
+                warning("FTP control connection closed. Trying to reopen.")
+                time.sleep(_sleep_between_retries)
+                self._FTP_instance = None
+                self.put(relpath, fp, mode, retries+1)
+
 
     def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
@@ -282,7 +358,11 @@
             f = self._get_FTP()
             return FtpStatResult(f, self._abspath(relpath))
         except ftplib.error_perm, e:
-            raise TransportError(orig_error=e)
+            if "no such file" in str(e).lower():
+                raise NoSuchFile("Error storing %s: %s"
+                                 % (self.abspath(relpath), str(e)), extra=e)
+            else:
+                raise FtpTransportError(orig_error=e)
 
     def lock_read(self, relpath):
         """Lock the given file for shared (read) access.

=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2005-12-20 02:45:27 +0000
+++ bzrlib/weave.py	2006-01-31 14:09:11 +0000
@@ -799,7 +799,7 @@
                        state
 
 
-    def join(self, other):
+    def join(self, other, pb=None, msg=None):
         import sys, time
         """Integrate versions from other into this weave.
 
@@ -809,6 +809,10 @@
 
         It is illegal for the two weaves to contain different values 
         or different parents for any version.  See also reweave().
+
+        :param other: The other weave to pull into this one
+        :param pb: An optional progress bar
+        :param msg: An optional message to display for progress
         """
         if other.numversions() == 0:
             return          # nothing to update, easy
@@ -818,6 +822,9 @@
         for other_idx, name in enumerate(other._names):
             self._check_version_consistent(other, other_idx, name)
 
+        if pb and not msg:
+            msg = 'weave join'
+
         merged = 0
         processed = 0
         time0 = time.time( )
@@ -829,14 +836,17 @@
             sha1 = other._sha1s[other_idx]
 
             processed += 1
-           
-            if name in self._names:
+
+            if name in self._name_map:
                 idx = self.lookup(name)
-                n1 = map(other.idx_to_name, other._parents[other_idx] )
-                n2 = map(self.idx_to_name, self._parents[other_idx] )
+                n1 = set(map(other.idx_to_name, other._parents[other_idx]))
+                n2 = set(map(self.idx_to_name, self._parents[idx]))
                 if sha1 ==  self._sha1s[idx] and n1 == n2:
                         continue
 
+            if pb:
+                pb.update(msg, other_idx, len(other._names))
+           
             merged += 1
             lines = other.get_lines(other_idx)
             self.add(name, new_parents, lines, sha1)
@@ -879,10 +889,8 @@
                                  % (name))
             self_parents = self._parents[this_idx]
             other_parents = other._parents[other_idx]
-            n1 = [self._names[i] for i in self_parents]
-            n2 = [other._names[i] for i in other_parents]
-            n1.sort()
-            n2.sort()
+            n1 = set([self._names[i] for i in self_parents])
+            n2 = set([other._names[i] for i in other_parents])
             if n1 != n2:
                 raise WeaveParentMismatch("inconsistent parents "
                     "for version {%s}: %s vs %s" % (name, n1, n2))
@@ -891,14 +899,19 @@
         else:
             return False
 
-    def reweave(self, other):
-        """Reweave self with other."""
-        new_weave = reweave(self, other)
+    def reweave(self, other, pb=None, msg=None):
+        """Reweave self with other.
+
+        :param other: The other weave to merge
+        :param pb: An optional progress bar, indicating how far done we are
+        :param msg: An optional message for the progress
+        """
+        new_weave = reweave(self, other, pb=pb, msg=msg)
         for attr in self.__slots__:
             setattr(self, attr, getattr(new_weave, attr))
 
 
-def reweave(wa, wb):
+def reweave(wa, wb, pb=None, msg=None):
     """Combine two weaves and return the result.
 
     This works even if a revision R has different parents in 
@@ -909,6 +922,9 @@
     might be possible but it should only be necessary to do 
     this operation rarely, when a new previously ghost version is 
     inserted.
+
+    :param pb: An optional progress bar, indicating how far done we are
+    :param msg: An optional message for the progress
     """
     wr = Weave()
     ia = ib = 0
@@ -920,11 +936,25 @@
     mutter("combined parents: %r", combined_parents)
     order = topo_sort(combined_parents.iteritems())
     mutter("order to reweave: %r", order)
-    for name in order:
+
+    if pb and not msg:
+        msg = 'reweave'
+
+    for idx, name in enumerate(order):
+        if pb:
+            pb.update(msg, idx, len(order))
         if name in wa._name_map:
             lines = wa.get_lines(name)
             if name in wb._name_map:
-                assert lines == wb.get_lines(name)
+                lines_b = wb.get_lines(name)
+                if lines != lines_b:
+                    mutter('Weaves differ on content. rev_id {%s}', name)
+                    mutter('weaves: %s, %s', wa._weave_name, wb._weave_name)
+                    import difflib
+                    lines = list(difflib.unified_diff(lines, lines_b,
+                            wa._weave_name, wb._weave_name))
+                    mutter('lines:\n%s', ''.join(lines))
+                    raise errors.WeaveTextDiffers(name, wa, wb)
         else:
             lines = wb.get_lines(name)
         wr.add(name, combined_parents[name], lines)

=== added file 'generate_docs.py'
--- generate_docs.py	1970-01-01 00:00:00 +0000
+++ generate_docs.py	2006-01-31 15:33:47 +0000
@@ -0,0 +1,93 @@
+#!/usr/bin/python
+
+# Copyright 2005 Canonical Ltd.
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+"""%(prog)s - generate information from built-in bzr help
+
+%(prog)s creates a file with information on bzr in one of
+several different output formats:
+
+    man              man page
+    bash_completion  bash completion script
+    ...
+
+Examples: 
+
+    python2.4 generated-docs.py man
+    python2.4 generated-docs.py bash_completion
+
+Run "%(prog)s --help" for the option reference.
+"""
+
+import sys
+from optparse import OptionParser
+
+import tools.doc_generate
+
+def main(argv):
+    parser = OptionParser(usage="%prog [options] OUTPUT_FORMAT")
+
+    parser.add_option("-s", "--show-filename",
+                      action="store_true", dest="show_filename", default=False,
+                      help="print default filename on stdout")
+
+    parser.add_option("-o", "--output", dest="filename", metavar="FILE",
+                      help="write output to FILE")
+
+    parser.add_option("-b", "--bzr-name",
+                      dest="bzr_name", default="bzr", metavar="EXEC_NAME",
+                      help="name of bzr executable")
+
+    parser.add_option("-e", "--examples",
+                      action="callback", callback=print_extended_help,
+                      help="Examples of ways to call generate_doc")
+
+
+    (options, args) = parser.parse_args(argv)
+
+    if len(args) != 2:
+        parser.print_help()
+        sys.exit(1)
+
+    infogen_type = args[1]
+    infogen_mod = tools.doc_generate.get_module(infogen_type)
+
+    if options.filename:
+        outfilename = options.filename
+    else:
+        outfilename = infogen_mod.get_filename(options)
+
+    if outfilename == "-":
+        outfile = sys.stdout
+    else:
+        outfile = open(outfilename,"w")
+
+    if options.show_filename and (outfilename != "-"):
+        print >>sys.stdout, outfilename
+    
+    infogen_mod.infogen(options, outfile)
+
+def print_extended_help(option, opt, value, parser):
+    """ Program help examples
+
+    Prints out the examples stored in the docstring. 
+
+    """
+    print >>sys.stdout, __doc__ % {"prog":sys.argv[0]}
+    sys.exit(0)
+
+if __name__ == '__main__':
+    main(sys.argv)

=== modified file 'setup.py'
--- setup.py	2005-12-24 02:20:45 +0000
+++ setup.py	2006-01-31 15:33:47 +0000
@@ -73,8 +73,8 @@
     def run(self):
         build.run(self)
 
-        import bzr_man
-        bzr_man.main()
+        import generate_docs
+        generate_docs.main(argv=["bzr", "man"])
 
 ########################
 ## Setup

=== added file 'tools/__init__.py'
=== added directory 'tools/doc_generate'
=== added file 'tools/doc_generate/__init__.py'
--- tools/doc_generate/__init__.py	1970-01-01 00:00:00 +0000
+++ tools/doc_generate/__init__.py	2006-01-31 15:51:02 +0000
@@ -0,0 +1,32 @@
+#!/usr/bin/python
+
+# Copyright 2005 Canonical Ltd.
+# Written by Hans Ulrich Niedermann
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""tools/doc_generate/__init__.py - main program for bzr information generation stuff"""
+
+import sys
+
+
+def get_module(target):
+    mod_name = "tools.doc_generate.autodoc_%s" % (target)
+    mod = __import__(mod_name)
+    components = mod_name.split('.')
+    for comp in components[1:]:
+        mod = getattr(mod, comp)
+    return mod
+

=== added file 'tools/doc_generate/autodoc_bash_completion.py'
--- tools/doc_generate/autodoc_bash_completion.py	1970-01-01 00:00:00 +0000
+++ tools/doc_generate/autodoc_bash_completion.py	2006-01-31 15:51:02 +0000
@@ -0,0 +1,54 @@
+# Copyright 2005 Canonical Ltd.
+# Written by Hans Ulrich Niedermann
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"bash_completion.py - create bash completion script from built-in bzr help"
+
+import os
+import sys
+import time
+
+import bzrlib
+import bzrlib.help
+import bzrlib.commands
+
+
+def get_filename(options):
+    return "%s.bash_completion" % (options.bzr_name)
+
+
+def infogen(options, outfile):
+    t = time.time()
+    tt = time.gmtime(t)
+    params = \
+           { "bzrcmd": options.bzr_name,
+             "datestamp": time.strftime("%Y-%m-%d",tt),
+             "timestamp": time.strftime("%Y-%m-%d %H:%M:%S +0000",tt),
+             "version": bzrlib.__version__,
+             }
+
+    outfile.write(preamble % params)
+
+
+preamble = """\
+# bash completion functions for for %(bzrcmd)s (bazaar-ng)
+#
+# Large parts of this file are autogenerated from the internal
+# bazaar-ng documentation and data structures.
+#
+# Generation time: %(timestamp)s
+"""
+

=== renamed file 'bzr_man.py' => 'tools/doc_generate/autodoc_man.py'
--- bzr_man.py	2005-12-08 12:01:09 +0000
+++ tools/doc_generate/autodoc_man.py	2006-01-31 15:51:02 +0000
@@ -1,7 +1,4 @@
-#!/usr/bin/python
-
-# Copyright (C) 2005 by Hans Ulrich Niedermann
-# Portions Copyright (C) 2005 by Canonical Ltd
+# Copyright 2005 Canonical Ltd.
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -17,21 +14,47 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-#<<< code taken from bzr (C) Canonical
-
-import os, sys
-
-import bzrlib, bzrlib.help
-
-#>>> code taken from bzr (C) Canonical
-
-#<<< code by HUN
-
+"""man.py - create man page from built-in bzr help and static text
+
+TODO:
+  * use usage information instead of simple "bzr foo" in COMMAND OVERVIEW
+  * add command aliases
+"""
+
+import os
+import sys
+import textwrap
 import time
-import re
+
+import bzrlib
+import bzrlib.help
+import bzrlib.commands
+
+
+def get_filename(options):
+    """Provides name of manpage"""
+    return "%s.1" % (options.bzr_name)
+
+
+def infogen(options, outfile):
+    """Assembles a man page"""
+    t = time.time()
+    tt = time.gmtime(t)
+    params = \
+           { "bzrcmd": options.bzr_name,
+             "datestamp": time.strftime("%Y-%m-%d",tt),
+             "timestamp": time.strftime("%Y-%m-%d %H:%M:%S +0000",tt),
+             "version": bzrlib.__version__,
+             }
+    outfile.write(man_preamble % params)
+    outfile.write(man_escape(man_head % params))
+    outfile.write(man_escape(getcommand_list(params)))
+    outfile.write(man_escape(getcommand_help(params)))
+    outfile.write(man_escape(man_foot % params))
 
 
 def man_escape(string):
+    """Escapes strings for man page compatibility"""
     result = string.replace("\\","\\\\")
     result = result.replace("`","\\`")
     result = result.replace("'","\\'")
@@ -39,96 +62,72 @@
     return result
 
 
-class Parser:
-
-    def parse_line(self, line):
-        pass
-
-
-class CommandListParser(Parser):
-
-    """Parser for output of "bzr help commands".
-
-    The parsed content can then be used to
-    - write a "COMMAND OVERVIEW" section into a man page
-    - provide a list of all commands
-    """
-
-    def __init__(self,params):
-        self.params = params
-        self.command_usage = []
-        self.all_commands = []
-        self.usage_exp = re.compile("([a-z0-9-]+).*")
-        self.descr_exp = re.compile("    ([A-Z].*)\s*")
-        self.state = 0
-        self.command = None
-        self.usage = None
-        self.descr = None
-
-    def parse_line(self, line):
-        m = self.usage_exp.match(line)
-	if line == '':
-		return
-        if m:
-            if self.state == 0:
-                if self.usage:
-                    self.command_usage.append((self.command,self.usage,self.descr))
-                    self.all_commands.append(self.command)
-                self.usage = " ".join(line.split(" ")[1:])
-                self.command = m.groups()[0]
-            else:
-                raise RuntimeError, "matching usage line in state %d" % state
-            self.state = 1
-            return
-        m = self.descr_exp.match(line)
-        if m:
-            if self.state == 1:
-                self.descr = m.groups()[0]
-            else:
-                raise RuntimeError, "matching descr line in state %d" % state
-            self.state = 0
-            return
-        raise RuntimeError, "Cannot parse this line ('%s')." % line
-
-    def end_parse(self):
-        if self.state == 0:
-            if self.usage:
-                self.command_usage.append((self.command,self.usage,self.descr))
-                self.all_commands.append(self.command)
+def command_name_list():
+    """Builds a list of command names from bzrlib"""
+    command_names = bzrlib.commands.builtin_command_names()
+    command_names.sort()
+    return command_names
+
+
+def getcommand_list (params):
+    """Builds summary help for command names in manpage format"""
+    bzrcmd = params["bzrcmd"]
+    output = '.SH "COMMAND OVERVIEW"\n'
+    for cmd_name in command_name_list():
+        cmd_object = bzrlib.commands.get_cmd_object(cmd_name)
+        if cmd_object.hidden:
+            continue
+        cmd_help = cmd_object.help()
+        if cmd_help:
+            firstline = cmd_help.split('\n', 1)[0]
+            usage = bzrlib.help.command_usage(cmd_object)
+            tmp = '.TP\n.B "%s"\n%s\n' % (usage, firstline)
+            output = output + tmp
         else:
-            raise RuntimeError, "ending parse in state %d" % state
-
-    def write_to_manpage(self, outfile):
-        bzrcmd = self.params["bzrcmd"]
-        outfile.write('.SH "COMMAND OVERVIEW"\n')
-        for (command,usage,descr) in self.command_usage:
-            outfile.write('.TP\n.B "%s %s"\n%s\n' % (bzrcmd, usage, descr))
-
-
-class HelpReader:
-
-    def __init__(self, parser):
-        self.parser = parser
-
-    def write(self, data):
-        if data[-1] == '\n':
-            data = data[:-1]
-        for line in data.split('\n'):
-            self.parser.parse_line(line)
-
-
-def write_command_details(params, command, usage, descr, outfile):
-    x = ('.SS "%s %s"\n.B "%s"\n.PP\n.B "Usage:"\n%s %s\n\n' %
-         (params["bzrcmd"],
-          command,
-          descr,
-          params["bzrcmd"],
-          usage))
-    outfile.write(man_escape(x))
+            raise RuntimeError, "Command '%s' has no help text" % (cmd_name)
+    return output
+
+
+def getcommand_help(params):
+    """Shows individual options for a bzr command"""
+    output='.SH "COMMAND REFERENCE"\n'
+    for cmd_name in command_name_list():
+        cmd_object = bzrlib.commands.get_cmd_object(cmd_name)
+        if cmd_object.hidden:
+            continue
+        output = output + format_command(params, cmd_object)
+    return output
+
+
+def format_command (params, cmd):
+    """Provides long help for each public command"""
+    subsection_header = '.SS "%s"\n' % (bzrlib.help.command_usage(cmd))
+    doc = "%s\n" % (cmd.__doc__)
+    docsplit = cmd.__doc__.split('\n')
+    doc = '\n'.join([docsplit[0]] + [line[4:] for line in docsplit[1:]])
+    option_str = ""
+    options = cmd.options()
+    if options:
+        option_str = "\nOptions:\n"
+        for option_name, option in sorted(options.items()):
+            l = '    --' + option_name
+            if option.type is not None:
+                l += ' ' + option.argname.upper()
+            short_name = option.short_name()
+            if short_name:
+                assert len(short_name) == 1
+                l += ', -' + short_name
+            l += (30 - len(l)) * ' ' + option.help
+            # TODO: Split help over multiple lines with
+            # correct indenting and wrapping.
+            wrapped = textwrap.fill(l, initial_indent='',
+                                    subsequent_indent=30*' ')
+            option_str = option_str + wrapped + '\n'       
+    return subsection_header + option_str + "\n" + doc + "\n"
 
 
 man_preamble = """\
-.\\\" Man page for %(bzrcmd)s (bazaar-ng)
+Man page for %(bzrcmd)s (bazaar-ng)
 .\\\"
 .\\\" Large parts of this file are autogenerated from the output of
 .\\\"     \"%(bzrcmd)s help commands\"
@@ -138,8 +137,7 @@
 .\\\"
 """
 
-# The DESCRIPTION text was taken from http://www.bazaar-ng.org/
-# and is thus (C) Canonical
+
 man_head = """\
 .TH bzr 1 "%(datestamp)s" "%(version)s" "bazaar-ng"
 .SH "NAME"
@@ -174,75 +172,24 @@
 is to look for external command.
 .TP
 .I "BZREMAIL"
-E-Mail address of the user. Overrides
-.I "~/.bzr.conf/email" and
-.IR "EMAIL" .
-Example content:
-.I "John Doe <john@example.com>"
+E-Mail address of the user. Overrides default user config.
 .TP
 .I "EMAIL"
-E-Mail address of the user. Overridden by the content of the file
-.I "~/.bzr.conf/email"
-and of the environment variable
-.IR "BZREMAIL" .
+E-Mail address of the user. Overriddes default user config.
 .SH "FILES"
 .TP
-.I "~/.bzr.conf/"
-Directory where all the user\'s settings are stored.
-.TP
-.I "~/.bzr.conf/email"
-Stores name and email address of the user. Overrides content of
-.I "EMAIL"
-environment variable. Example content:
-.I "John Doe <john@example.com>"
+.I "~/.bazaar/bazaar.conf/"
+Contains the default user config. Only one section, [DEFAULT] is allowed. A 
+typical default config file may be similiar to:
+.br
+.br
+.B [DEFAULT]
+.br
+.B email=John Doe <jdoe@isp.com>
 .SH "SEE ALSO"
+.UR http://bazaar.canonical.com/
+.BR http://bazaar.canonical.com/,
 .UR http://www.bazaar-ng.org/
-.BR http://www.bazaar-ng.org/,
-.UR http://www.bazaar-ng.org/doc/
-.BR http://www.bazaar-ng.org/doc/
+.BR http://www.bazaar-ng.org/
 """
 
-def main(args=[]):
-    """ main function
-    :param  args:   command-line arguments (sys.argv[1:])
-    :type   args:   list
-    """
-    t = time.time()
-    tt = time.gmtime(t)
-    params = \
-           { "bzrcmd": "bzr",
-             "datestamp": time.strftime("%Y-%m-%d",tt),
-             "timestamp": time.strftime("%Y-%m-%d %H:%M:%S +0000",tt),
-             "version": bzrlib.__version__,
-             }
-
-    clp = CommandListParser(params)
-    bzrlib.help.help("commands", outfile=HelpReader(clp))
-    clp.end_parse()
-
-    filename = "bzr.1"
-    if len(args) == 1:
-        filename = args[0]
-    if filename == "-":
-        outfile = sys.stdout
-    else:
-        outfile = open(filename,"w")
-
-    outfile.write(man_preamble % params)
-    outfile.write(man_escape(man_head % params))
-    clp.write_to_manpage(outfile)
-
-    # FIXME:
-    #   This doesn't do more than the summary so far.
-    #outfile.write('.SH "DETAILED COMMAND DESCRIPTION"\n')
-    #for (command,usage,descr) in clp.command_usage:
-    #    write_command_details(params, command, usage, descr, outfile = outfile)
-
-    outfile.write(man_escape(man_foot % params))
-
-
-if __name__ == '__main__':
-    main(sys.argv[1:])
-
-
-#>>> code by HUN

